"""
Google Chat Examples Package
Contains example configurations and usage patterns for enhanced Google Chat functionality.
"""

from .card_examples import (
    FORM_CARD_EXAMPLE,
    INTERACTIVE_CARD_EXAMPLE,
    NOTIFICATION_CARD_EXAMPLE,
    SIMPLE_CARD_EXAMPLE,
    TEST_SCENARIOS,
    WELCOME_CARD_EXAMPLE,
    get_example_card,
    get_test_scenario,
    list_available_examples,
)

__all__ = [
    "SIMPLE_CARD_EXAMPLE",
    "INTERACTIVE_CARD_EXAMPLE",
    "FORM_CARD_EXAMPLE",
    "NOTIFICATION_CARD_EXAMPLE",
    "WELCOME_CARD_EXAMPLE",
    "TEST_SCENARIOS",
    "get_example_card",
    "get_test_scenario",
    "list_available_examples",
]
